//
// UINode.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation
import SwiftUI


public struct CVUUINode: Equatable, Identifiable, Codable {
    var type: CVUUIElementFamily
    var children: [CVUUINode] = []
    var properties: [String: CVUValue] = [:]

    public private(set) var id = UUID()
}

extension CVUUINode: CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String {
        let tabs = Array(repeating: tab, count: depth).joined()
        
        guard !properties.isEmpty || !children.isEmpty else { return "\(includeInitialTab ? tabs : "")\(type)" }
        
        let propertiesString = properties.isEmpty ? "" : properties.toCVUString(depth: depth + 1, tab: tab, includeInitialTab: true).appending("\n")
        
        let childrenString = children.isEmpty ? "" : children.map { node in
            node.toCVUString(depth: depth + 1, tab: tab, includeInitialTab: true)
        }.joined(separator: "\n\n").appending("\n")
        
        let includeNewLineBetweenPropertiesAndChildren = !propertiesString.isEmpty && !childrenString.isEmpty
        
        return "\(includeInitialTab ? tabs : "")\(type) {\n\(propertiesString)\(includeNewLineBetweenPropertiesAndChildren ? "\n" : "")\(childrenString)\(tabs)}"
    }

    public var description: String {
        toCVUString(depth: 0, tab: "    ", includeInitialTab: true)
    }
}
