//
//  CVUValue_Constant.swift
//  Memri
//
//  Created by T Brennan on 28/1/21.
//

import Foundation

extension CVUValue {
    enum Constant: Equatable, AutoCodable {
        case argument(String)
        case number(Double)
        case string(String)
        case bool(Bool)
        case colorHex(String)
        case `nil`
        
        func asString() -> String {
            switch self {
            case .argument(let x):
                return x
            case .number(let x):
                return "\(x)"
            case .string(let x):
                return x
            case .colorHex(let x):
                return x
            case .bool(let x):
                return "\(x)"
            case .nil:
                return ""
            }
        }
        
        func asNumber() -> Double? {
            switch self {
            case .argument:
                return nil
            case .number(let x):
                return x
            case .string(let x):
                return Double(x)
            case .bool(let x):
                return x ? 1 : 0
            case .colorHex:
                return nil
            case .nil:
                return nil
            }
        }
        
        func asBool() -> Bool? {
            switch self {
            case .argument:
                return nil
            case .number(let x):
                // Number greater than zero = true
                return x > 0
            case .string(let x):
                /// Non-empty string = true (unless string is `false`)
                return x != "" && x != "false"
            case .bool(let x):
                return x
            case .colorHex:
                return nil
            case .nil:
                return nil
            }
        }
    }
}


extension CVUValue.Constant {
    func toCVUString(insideStringMode: Bool = false) -> String {
        switch self {
        case .argument(let arg):
            return arg
        case .bool(let bool):
            return bool ? "true" : "false"
        case .number(let number):
            return number.format("0.##")
        case .string(let string):
            let escaped = string.regexReplace(pattern: "\"", with: "\\\\\"") // Escape `"` characters
            return insideStringMode ? escaped : "\"\(escaped)\""
        case .colorHex(let hex):
            return "#\(hex)"
        case .nil:
            return "nil"
        }
    }
}
