//
// CVUToString.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation
import SwiftUI

protocol CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String
}

extension Dictionary: CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String {
        let strings = self.compactMap { key, value -> String? in
            guard let value = value as? CVUStringConvertible else { return nil }
            return "\(key): \(value.toCVUString(depth: depth, tab: tab, includeInitialTab: false))"
        }.sorted()
        let tabs = Array(repeating: tab, count: depth).joined()
        return "\(includeInitialTab ? tabs : "")\(strings.joined(separator: "\n\(tabs)"))"
    }
}

extension Array: CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String {
        let strings = self.compactMap { value -> String? in
            guard let value = value as? CVUStringConvertible else { return nil }
            return value.toCVUString(depth: depth + 1, tab: tab, includeInitialTab: false)
        }
        guard !strings.isEmpty else {
            return "[]"
        }
        let isMultiline = strings.contains(where: { $0.contains("\n") })
        let tabs = Array<String>(repeating: tab, count: depth).joined()
        let tabsPlus = Array<String>(repeating: tab, count: depth + 1).joined()
        return "\((isMultiline && includeInitialTab) ? tabs : "")\(isMultiline ? "[\n\(tabsPlus)" : "")\(strings.joined(separator: isMultiline ? "\n\(tabsPlus)" : " "))\(isMultiline ? "\n\(tabs)]" : "")"
    }
}

extension Optional: CVUStringConvertible where Wrapped: CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String {
        let tabs = Array<String>(repeating: tab, count: depth).joined()
        switch self {
        case .none:
            return "\(includeInitialTab ? tabs : "")nil"
        case .some(let value):
            return value.toCVUString(depth: depth, tab: tab, includeInitialTab: includeInitialTab)
        }
    }
}
