//
//  PodAPIPayloads.swift
//  MemriDatabase
//
//  Created by T Brennan on 19/12/20.
//

import Foundation

/// A namespace for types representing Pod API payloads. Refer to pod API documentation for expected payloads for each API.
enum PodAPIPayload {
    struct EmptyPayload: Encodable {
        func encode(to encoder: Encoder) throws {
            var container = encoder.singleValueContainer()
            try container.encodeNil()
        }
    }
    
    struct FileSHA:  Encodable {
        var sha: String
    }
    struct ItemUID:  Encodable {
        var uid: StringUUID
    }

    struct ItemUIDList: Encodable {
        var uids: Set<StringUUID>
        
        func encode(to encoder: Encoder) throws {
           var container = encoder.singleValueContainer()
            try container.encode(uids)
        }
    }
    
    struct ItemUIDWithServicePayload:  Encodable {
        var uid: StringUUID
        var servicePayload: ServicePayload
    }
    
    /// Some of the pod APIs require a `servicePayload` included in the main payload
    struct ServicePayload: Encodable {
        var databaseKey: String
        var ownerKey: String
        
        init(connectionConfig: PodAPIConnectionDetails) {
            self.databaseKey = connectionConfig.databaseKey
            self.ownerKey = connectionConfig.ownerKey
        }
    }
    
    struct BulkAction: Encodable {
        var createItems: [[String: AnyEncodable]]
        var updateItems: [[String: AnyEncodable]]
        var deleteItems: [StringUUID]
        var createEdges: [[String: AnyEncodable]]
        var updateEdges: [[String: AnyEncodable]]
        var deleteEdges: [[String: AnyEncodable]]
    }
}
