//
//  ItemEdgeRecord.swift
//  MemriDatabase
//
//  Created by T Brennan on 21/12/20.
//

import Foundation
import GRDB

class ItemEdgeRecord: BaseRecord {
    static override var databaseTableName: String { "itemEdge" }
    
    let sourceUID: StringUUID
    let name: String
    let targetUID: StringUUID
    
    enum Columns: String, ColumnExpression {
        case sourceUID, name, targetUID
    }
    
    static var ownerForeignKey = ForeignKey([Columns.sourceUID], to: [ItemRecord.Columns.uid])
    static var owningItem = hasOne(ItemRecord.self, using: ownerForeignKey)
    
    static var targetForeignKey = ForeignKey([Columns.targetUID], to: [ItemRecord.Columns.uid])
    static var targetItem = hasOne(ItemRecord.self, using: targetForeignKey)

    func owningItem(db: Database) -> ItemRecord? {
        try? ItemRecord.filter(key: sourceUID).fetchOne(db)
    }
    
    func targetItem(db: Database) -> ItemRecord? {
        try? ItemRecord.filter(key: targetUID).fetchOne(db)
    }
    
    internal init(sourceUID: StringUUID, name: String, targetUID: StringUUID) {
        self.sourceUID = sourceUID
        self.name = name
        self.targetUID = targetUID
        super.init()
    }
    
    required init(row: Row) {
        sourceUID = row[Columns.sourceUID]
        name = row[Columns.name]
        targetUID = row[Columns.targetUID]
        super.init(row: row)
    }
    
    override func encode(to container: inout PersistenceContainer) {
        container[Columns.sourceUID] = sourceUID
        container[Columns.name] = name
        container[Columns.targetUID] = targetUID
    }
    
    func _syncDict() -> [String: AnyEncodable] {
        let keyProperties: [String: AnyEncodable] = [
            "_type": name,
            "_source": sourceUID,
            "_target": targetUID
        ].mapValues { AnyEncodable($0) }

        return keyProperties
    }
    
    func save(db dbController: DatabaseController = AppController.shared.databaseController) throws {
        try dbController.writeSync { (db) in
            try save(db)
        }
    }
}
