//
//  ItemPropertyRecord.swift
//  MemriDatabase
//
//  Created by T Brennan on 21/12/20.
//

import Foundation
import GRDB

class ItemPropertyRecord: BaseRecord, Identifiable {
    static override var databaseTableName: String { "itemProperty" }
    static var databaseSearchTableName: String { "itemProperty_search" }
    
    let itemUID: StringUUID
    let name: String
    
    var value: DatabaseValue
    func value(
        itemType: String,
        schema: Schema = AppController.shared.databaseController.schema
    ) -> PropertyDatabaseValue? {
        guard let expectedType = schema.types[itemType]?.propertyTypes[name]?.valueType else {
            return nil
        }
        return PropertyDatabaseValue(databaseValue: value, propertyType: expectedType)
    }
    
    enum Columns: String, ColumnExpression {
        case itemUID, name, value
    }
    
    var id: Int {
        var hasher = Hasher()
        hasher.combine(itemUID)
        hasher.combine(name)
        return hasher.finalize()
    }
    
    internal init(itemUID: StringUUID, name: String, value: PropertyDatabaseValue) {
        self.itemUID = itemUID
        self.name = name
        self.value = value.databaseValue
        super.init()
    }
    
    required init(row: Row) {
        itemUID = row[Columns.itemUID]
        name = row[Columns.name]
        value = row[Columns.value]
        super.init(row: row)
    }
    
    override func encode(to container: inout PersistenceContainer) {
        container[Columns.itemUID] = itemUID
        container[Columns.name] = name
        container[Columns.value] = value
    }
    
    func save(db dbController: DatabaseController = AppController.shared.databaseController) throws {
        try dbController.writeSync { (db) in
            try save(db)
        }
    }
}
