//
//  String.swift
//  MemriDatabase
//
//  Created by T Brennan on 6/12/20.
//

import Foundation

extension String {
    // Return nil if string is only whitespace
    var nilIfBlank: String? {
        guard contains(where: { !$0.isWhitespace }) else { return nil }
        return self
    }
    
    // Return true if the string only contains whitespace characters
    var isOnlyWhitespace: Bool {
        !contains { !$0.isWhitespace }
    }
    
    func regexReplace(pattern: String, with: String = "") -> String {
        do {
            let regex = try NSRegularExpression(
                pattern: pattern,
                options: NSRegularExpression.Options.caseInsensitive
            )
            let range = NSMakeRange(0, count)
            return regex.stringByReplacingMatches(
                in: self,
                options: [],
                range: range,
                withTemplate: with
            )
        }
        catch {
            return self
        }
    }
    
    func camelCaseToWords() -> String {
        unicodeScalars.reduce("") {
            if CharacterSet.uppercaseLetters.contains($1) {
                return ($0 + " " + String($1).lowercased())
            }
            else {
                return $0 + String($1)
            }
        }.capitalizingFirst()
    }
}

extension String {
    func strippingHTMLtags() -> String {
        HTMLHelper.getPlainText(html: self)
            .trimmingCharacters(in: .newlines) // Remove newlines at start/end
    }
}

// MARK: Capitalisation
extension String {
    public func titleCase() -> String {
        split { $0.isWhitespace }.map { $0.capitalizingFirst() }.joined(separator: " ")
    }
}

extension RangeReplaceableCollection where Element == Character {
    func capitalizingFirst() -> String {
        guard let first = first else { return String(self) }
        return first.uppercased() + dropFirst()
    }
}
