//
//  SwiftUI.View.swift
//  MemriDatabase
//
//  Created by T Brennan on 14/12/20.
//

import Foundation
import SwiftUI

extension View {
    @ViewBuilder
    func `showIf`(_ conditional: Bool) -> some View {
        if conditional {
            self
        }
    }
    
    @ViewBuilder
    func `if`<Content: View>(_ conditional: Bool, content: (Self) -> Content) -> some View {
        if conditional {
            content(self)
        }
        else {
            self
        }
    }

    @ViewBuilder
    func ifLet<T, Content: View>(_ optional: T?, content: (Self, T) -> Content) -> some View {
        if optional != nil {
            optional.map { content(self, $0) }
        }
        else {
            self
        }
    }
}

extension View {
    func eraseToAnyView() -> AnyView {
        AnyView(self)
    }
}
