//
// CVU_Button.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying a button
/// - Use the `onPress` property to provide a CVU Action for the button
struct CVU_Button: View {
    var nodeResolver: CVUUINodeResolver
    @EnvironmentObject var sceneController: SceneController

    func onPress() {
        guard let action = self.nodeResolver.propertyResolver.action(key: "onPress") else { return }
        action.execute(sceneController: sceneController, context: nodeResolver.context)
    }
    
    var body: some View {
        Button(action: onPress) {
            self.nodeResolver.childrenInForEach()
        }
        .buttonStyle(Style())
    }

    struct Style: ButtonStyle {
        func makeBody(configuration: Configuration) -> some View {
            configuration
                .label
                .shadow(radius: configuration.isPressed ? 4 : 0)
        }
    }
}
