//
//  CVU_ForEach.swift
//  Memri
//
//  Created by T Brennan on 30/1/21.
//

import Foundation
import SwiftUI

/// A CVU element for displaying a child for each item in an array
/// - use this inside a HStack or VStack
/// - set the `items` property to an expression resolving to a list of items
/// - any child elements will be repeated for each item. The `.` in any expressions will resolve to each particular item being shown within the ForEach
struct CVU_ForEach: View {
    var nodeResolver: CVUUINodeResolver

    var body: some View {
        let items = nodeResolver.propertyResolver.items("items")
        if !items.isEmpty {
            ForEach(items) { item in
                let context = nodeResolver.context.replacingItem(item)
                ForEach(nodeResolver.node.children) { child in
                    CVUElementView(nodeResolver: CVUUINodeResolver(context: context, lookup: nodeResolver.lookup, node: child, db: nodeResolver.db))
                }
            }
        }
    }
}
