//
// CVU_Stack.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying multiple children in a Horizontal Stack
/// - set the `alignment` property to `top`, `center`, or `bottom`
/// - set the `fillWidth` property to `true` if the stack should fill all available width
struct CVU_HStack: View {
    var nodeResolver: CVUUINodeResolver

    var body: some View {
        HStack(alignment: nodeResolver.propertyResolver.alignment().vertical, spacing: nodeResolver.propertyResolver.spacing?.x) {
            nodeResolver.childrenInForEach()
        }
        .if(nodeResolver.propertyResolver.bool("fillWidth", defaultValue: false)) {
            $0.frame(maxWidth: .infinity, alignment: nodeResolver.propertyResolver.alignment())
        }
    }
}

/// A CVU element for displaying multiple children in a Vertical Stack
/// - set the `alignment` property to `leading`, `center`, or `trailing`
/// - set the `fillHeight` property to `true` if the stack should fill all available height
struct CVU_VStack: View {
    var nodeResolver: CVUUINodeResolver

    var body: some View {
        VStack(alignment: nodeResolver.propertyResolver.alignment().horizontal, spacing: nodeResolver.propertyResolver.spacing?.y) {
            nodeResolver.childrenInForEach()
        }
        .if(nodeResolver.propertyResolver.bool("fillHeight", defaultValue: false)) {
            $0.frame(maxHeight: .infinity, alignment: nodeResolver.propertyResolver.alignment())
        }
    }
}

/// A CVU element for displaying multiple children overlayed on each-other
/// - set the `alignment` property to align to any corner, edge, or center
struct CVU_ZStack: View {
    var nodeResolver: CVUUINodeResolver

    var body: some View {
        ZStack { //(alignment: nodeResolver.propertyResolver.alignment())
            nodeResolver.childrenInForEach()
        }
    }
}
