//
//  CVU_TextField.swift
//  Memri
//
//  Created by T Brennan on 30/1/21.
//

import Foundation
import SwiftUI

/// A CVU element for displaying an editable textfield
/// - Set the `value` property to an expression representing the item property to be edited
/// - Set the `hint` property to change the text displayed when the field is empty
/// - Set the `secure` to `true` to obscure content (eg. password field)
/// - Set the `color` property to change text color
struct CVU_TextField: View {
    var nodeResolver: CVUUINodeResolver
    
    @EnvironmentObject var sceneController: SceneController

    // Secure mode hides the input (eg. for passwords)
    var secureMode: Bool {
        nodeResolver.propertyResolver.bool("secure", defaultValue: false)
    }

    var hint: String? {
        nodeResolver.propertyResolver.string("hint")?.nilIfBlank
    }

    var contentBinding: Binding<String?> {
        nodeResolver.propertyResolver.binding(for: "value", defaultValue: nil) ?? .constant(nil)
    }

    var body: some View {
        MemriTextField(
            value: contentBinding,
            placeholder: hint,
            textColor: nodeResolver.propertyResolver.color()?.uiColor,
            isEditing: $sceneController.isInEditMode,
            isSharedEditingBinding: true,
            secureMode: secureMode
        )
    }
}
