//
// CVU_TimelineItem.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying a timeline-style item tag
/// - Set the `title`, `subtitle`, and `icon` properties
struct CVU_TimelineItem: View {
    var nodeResolver: CVUUINodeResolver

    var body: some View {
        TimelineItemView(
            icon: Image(systemName: nodeResolver.propertyResolver.string("icon") ?? "arrowtriangle.right"),
            title: nodeResolver.propertyResolver.string("title") ?? "-",
            subtitle: nodeResolver.propertyResolver.string("text"),
            backgroundColor: .gray
        )
    }
}


struct TimelineItemView: View {
    var icon = Image(systemName: "paperplane")
    var title: String = "Hello world"
    var subtitle: String? = nil
    var cornerRadius: CGFloat = 5

    var backgroundColor = Color(.systemGreen)
    var foregroundColor: Color {
        Color.white
    }

    var body: some View {
        VStack(alignment: .leading) {
            HStack(alignment: .lastTextBaseline) {
                icon
                Text(title)
                    .bold()
                    .lineLimit(1)
            }
            .font(.headline)
            subtitle.map {
                Text($0)
                    .font(.caption)
                    .lineLimit(2)
            }
        }
        .fixedSize(horizontal: false, vertical: true)
        .padding(5)
        .foregroundColor(foregroundColor)
        .background(backgroundColor)
        .mask(
            RoundedRectangle(cornerRadius: cornerRadius, style: .continuous)
        )
    }
}

struct TimelineItemView_Previews: PreviewProvider {
    static var previews: some View {
        TimelineItemView()
    }
}
