//
// BottomBar.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// This view contains the contents of the 'Bottom bar' which is shown below renderer content.
struct BottomBarView: View {
    @ObservedObject var context: ViewContextController

    var onSearchPressed: () -> Void
    var onFilterButtonPressed: () -> Void

    var currentFilter: String? {
        context.searchString
    }

    @ViewBuilder
    var body: some View {
        VStack(spacing: 0) {
            Divider()
            HStack(spacing: 4) {
                HStack(spacing: 4) {
                    Button(action: onSearchPressed) {
                        HStack(spacing: 0) {
                            Image(systemName: "magnifyingglass")
                                .padding(.trailing, 7)
                            if let filter = currentFilter {
                                Text(filter).font(.caption)
                                    .foregroundColor(Color(.label))
                            }
                        }
                        .padding([.leading, .vertical], 10)
                        .contentShape(Rectangle())
                    }
                    if currentFilter != nil {
                        Button {
                            context.searchString = nil
                        } label: {
                            Image(systemName: "clear")
                                .foregroundColor(Color(.label))
                                .font(.caption)
                        }
                    }
                }
                Spacer()
                Button(action: onFilterButtonPressed) {
                    Image(systemName: "line.horizontal.3.decrease.circle")
                    .padding(10)
                    .contentShape(Rectangle())
                }
            }
            .padding(.horizontal, 10)
            .font(Font.system(size: 20, weight: .medium))
            .background(Color(.secondarySystemBackground).edgesIgnoringSafeArea(.bottom))
        }
    }
}
