//
// BottomBar.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// This view provides the 'Navigation Bar' for the app interface
struct TopBarView: View {
    @ObservedObject
    var sceneController: SceneController

    @ViewBuilder
    var body: some View {
        VStack(spacing: 0) {
            HStack(spacing: 4) {
                HStack {
                    Button(action: {
                        withAnimation {
                            sceneController.navigationIsVisible.toggle()
                        }
                    }) {
                        Image(systemName: "line.horizontal.3")
                            .rotationEffect(sceneController.navigationIsVisible ? Angle(degrees: 90) : .zero)
                            .padding(10)
                            .contentShape(Rectangle())
                    }
                    if sceneController.canNavigateBack {
                        Button(action: {
                            sceneController.navigateBack()
                        }) {
                            Image(systemName: "chevron.backward.circle.fill")
                                .rotationEffect(sceneController.navigationIsVisible ? Angle(degrees: 90) : .zero)
                                .padding(10)
                                .contentShape(Rectangle())
                        }
                    }
                }
                .frame(minWidth: 100, alignment: .leading)
                if let context = sceneController.topMostContext,
                   let title = context.viewDefinitionPropertyResolver.string("title") {
                    Text(title).font(.headline).frame(maxWidth: .infinity)
                } else {
                    Spacer()
                }
                HStack {}
                    .frame(minWidth: 100, alignment: .trailing)
                }
                .padding(.horizontal, 10)
            .font(Font.system(size: 20, weight: .medium))
            .frame(minHeight: 48)
            .background(Color(.secondarySystemBackground).edgesIgnoringSafeArea(.top))
            
            Divider()
        }
    }
}
