//
// BarChartView.swift
// Copyright © 2020 memri. All rights reserved.

import Charts
import Foundation
import SwiftUI

struct PieChartSwiftUIView: UIViewRepresentable {
    var model: ChartModelLabelledY
    var holeColor: CVUColor? = nil
    var onPress: ((Int) -> Void)?

    func makeUIView(context: Context) -> PieChartView {
        let chartView = PieChartView()
        chartView.delegate = context.coordinator

        chartView.legend.enabled = false
        
        chartView.drawHoleEnabled = holeColor != nil
        chartView.holeColor = holeColor?.uiColor

        return chartView
    }

    func updateUIView(_ chartView: PieChartView, context _: Context) {
        chartView.data = model.generatePieChartData()
        chartView.notifyDataSetChanged()
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(self)
    }

    // MARK: - Implementing ChartViewDelegate

    final class Coordinator: NSObject, ChartViewDelegate {
        var parent: PieChartSwiftUIView

        init(_ parent: PieChartSwiftUIView) {
            self.parent = parent
        }

        // Fire when double-tapped
        func chartValueDoubleTapped(
            _: ChartViewBase,
            entry: ChartDataEntry,
            dataset _: Int,
            index _: Int
        ) {
            // Pressed twice
            guard let info = entry.data as? ChartEntryInfo else { return }
            parent.onPress?(info.dataIndex) // Using this index in case we have sorted the items
        }
    }
}
