//
//  GridRendererSettingsView.swift
//  Memri
//
//  Created by T Brennan on 3/2/21.
//

import Foundation
import SwiftUI

struct GridRendererSettingsView: View {
    @ObservedObject var context: ViewContextController
    
    var layoutBinding: Binding<String> {
        Binding<String>(get: {
            context.rendererDefinitionPropertyResolver.string("layout") ?? "grid"
        }, set: { newValue in
            context.setRendererProperty(renderer: "grid", property: "layout", value: .constant(.argument(newValue)))
        })
    }
    
    var scrollDirectionBinding: Binding<String> {
        Binding<String>(get: {
            context.rendererDefinitionPropertyResolver.string("scrollDirection") ?? "vertical"
        }, set: { newValue in
            context.setRendererProperty(renderer: "grid", property: "scrollDirection", value: .constant(.argument(newValue)))
        })
    }
    
    var body: some View {
        Picker("Layout", selection: layoutBinding) {
            Group {
                Text("Grid").tag("grid")
                Text("Photo grid").tag("photoGrid")
                Text("Waterfall").tag("waterfall")
            }.navigationBarTitle("Layout", displayMode: .inline)
        }
        if layoutBinding.wrappedValue == "grid" {
            Picker("Scroll direction", selection: scrollDirectionBinding) {
                Group {
                    Text("Vertical").tag("vertical")
                    Text("Horizontal").tag("horizontal")
                }.navigationBarTitle("Scroll direction", displayMode: .inline)
            }
        }
    }
}
