//
//  TimelineRendererSettingsView.swift
//  Memri
//
//  Created by T Brennan on 31/1/21.
//

import SwiftUI

struct TimelineRendererSettingsView: View {
    @ObservedObject var context: ViewContextController
    
    typealias DetailLevel = TimelineRendererModel.DetailLevel
    
    var detailLevelBinding: Binding<DetailLevel> {
        Binding<DetailLevel>(get: {
            context.rendererDefinitionPropertyResolver.string("detailLevel").flatMap(TimelineRendererModel.DetailLevel.init) ?? .day
        }, set: { newValue in
            context.setRendererProperty(renderer: "timeline", property: "detailLevel", value: .constant(.argument(newValue.rawValue)))
        })
    }
    
    var body: some View {
        Picker("Detail level", selection: detailLevelBinding) {
            Group {
                Text("Year").tag(DetailLevel.year)
                Text("Month").tag(DetailLevel.month)
                Text("Day").tag(DetailLevel.day)
                Text("Hour").tag(DetailLevel.hour)
            }.navigationBarTitle("Detail level", displayMode: .inline)
        }
    }
}
