//
// GeneralEditorRenderer.swift
// Copyright © 2020 memri. All rights reserved.

import Combine
import Foundation
import SwiftUI

//class GeneralEditorRendererController: RendererController, ObservableObject {
//    static let rendererType = RendererType(
//        name: "generalEditor",
//        icon: "pencil.circle.fill",
//        makeController: GeneralEditorRendererController.init,
//        makeConfig: GeneralEditorRendererController.makeConfig
//    )
//
//    required init(context: MemriContext, config: CascadingRendererConfig?) {
//        self.context = context
//        self.config = (config as? GeneralEditorRendererConfig) ?? GeneralEditorRendererConfig()
//    }
//
//    let context: MemriContext
//    let config: GeneralEditorRendererConfig
//
//    func makeView() -> AnyView {
//        GeneralEditorRendererView(controller: self).eraseToAnyView()
//    }
//
//    func update() {
//        objectWillChange.send()
//    }
//
//    static func makeConfig(
//        head: CVUParsedDefinition?,
//        tail: [CVUParsedDefinition]?,
//        host: Cascadable?
//    ) -> CascadingRendererConfig {
//        GeneralEditorRendererConfig(head, tail, host)
//    }
//}
//
//class GeneralEditorRendererConfig: CascadingRendererConfig, ConfigurableRenderConfig {
//    var type: String? = "generalEditor"
//
//    var layout: [GeneralEditorLayoutItem] {
//        cascadeList(
//            "layout",
//            uniqueKey: { $0["section"] as? String ?? "" },
//            merging: { old, new in
//                var result = old
//                for (key, value) in new {
//                    if old[key] == nil { result[key] = value }
//                    else if key == "exclude" {
//                        if var dict = old[key] as? [String] {
//                            result[key] = dict.append(contentsOf: new[key] as? [String] ?? [])
//                        }
//                    }
//                }
//
//                return result
//            }
//        )
//        .map { dict -> GeneralEditorLayoutItem in
//            GeneralEditorLayoutItem(
//                id: dict["section"] as? String ?? "",
//                dict: dict,
//                viewArguments: self.viewArguments
//            )
//        }
//    }
//
//    var showSortInConfig: Bool = false
//
//    var showContextualBarInEditMode: Bool = false
//
//    func configItems(context: MemriContext) -> [ConfigPanelModel.ConfigItem] {
//        []
//    }
//}

struct GeneralEditorRendererView: View {
    @EnvironmentObject var sceneController: SceneController
    @ObservedObject var viewContext: ViewContextController

    var name: String = "generalEditor"

    @ViewBuilder
    var stackContent: some View {
        let layout = viewContext.rendererDefinitionPropertyResolver.resolverArray("layout")
        let usedFields = getUsedFields(layout)

        if layout.count > 0 {
            ForEach(layout, id: \.properties) { layoutSection in
                GeneralEditorSection(
                    viewContext: viewContext,
                    layoutSection: layoutSection,
                    usedFields: usedFields
                )
            }
        }
    }

    var body: some View {
        ScrollView(.vertical) {
            VStack(alignment: .leading, spacing: 0) {
                stackContent
            }
        }
    }

    func getUsedFields(_ layout: [CVUPropertyResolver]) -> [String] {
        var result = [String]()
        for item in layout {
            let include:[String] = item.stringArray("fields")
            let exclude:[String] = item.stringArray("exclude")
            
            if include.count > 0 {
                result.append(contentsOf: include)
            }
            if exclude.count > 0 {
                result.append(contentsOf: exclude)
            }
        }
        return result
    }
}

// struct GeneralEditorView_Previews: PreviewProvider {
//    static var previews: some View {
//        let context = try! RootContext(name: "").mockBoot()
//
//        return ZStack {
//            VStack(alignment: .center, spacing: 0) {
//                TopNavigation()
//                GeneralEditorRendererView(controller: GeneralEditorRendererController(context: context, config: nil))
//                Search()
//            }.fullHeight()
//
//            ContextPane()
//        }.environmentObject(context)
//    }
// }

struct GeneralEditorSection: View {
    @EnvironmentObject var sceneController: SceneController
    @ObservedObject var viewContext: ViewContextController

    var item: ItemRecord?
    var layoutSection: CVUPropertyResolver
    var usedFields: [String]

    var body: some View {
        let resolver = viewContext.rendererDefinitionPropertyResolver
        let editMode = sceneController.isInEditMode
        let fields: [String] = (layoutSection.string("fields") == "*"
            ? getProperties(item, usedFields)
            : layoutSection.stringArray("fields"))
        let edgeNames = layoutSection.stringArray("edges")
        let edgeType = layoutSection.string("type")
        let edges = layoutSection.stringArray("edges") // , [Edge].self, item) ?? []
        let groupKey = layoutSection.string("section") ?? "" // , String.self) ?? ""

        let sectionStyle = self.sectionStyle(groupKey)
        let readOnly = self.layoutSection.bool("readOnly") // , Bool.self) ?? false
        let isEmpty = edges.count == 0 && fields.count == 0 && !editMode // self.layoutSection.has("edges") &&
        let hasGroup = viewContext.rendererDefinitionPropertyResolver.hasSubdefinition(groupKey) ?? false

        let title = (hasGroup ? sectionStyle.title : nil) ?? groupKey.camelCaseToWords()
            .uppercased()
        let dividers = sectionStyle.dividers ?? !(sectionStyle.showTitle ?? false)
        let showTitle = sectionStyle.showTitle ?? true
        let action = editMode
            ? sectionStyle
            .action ?? (!readOnly && edgeType != nil /* TODO: support multiple / many types*/
                ? getAction(edgeType: edgeNames[0], itemType: edgeType ?? "")
                : nil)
            : nil
        let spacing = sectionStyle.spacing ?? 0
        let padding = sectionStyle.padding

        return Section(
            header: Group {
                if showTitle && !isEmpty {
                    HStack(alignment: .bottom) {
                        Text(title)
                            .generalEditorHeader()

                        if action != nil {
                            Spacer()
                            // NOTE: Allowed force unwrapping
                            ActionButton(action: action, item: item)
                                .foregroundColor(Color(hex: "#777"))
                                .font(.system(size: 18, weight: .semibold))
                                .padding(.bottom, 10)
                        }
                    }.padding(.trailing, 20)
                }
                else {
                    EmptyView()
                }
            },

            content: {
                if isEmpty {
                    EmptyView()
                }
                else {
                    if dividers { Divider() }

                    // If a render group is defined in the render config
                    if hasGroup {
                        // Add spacing between the render elements
                        VStack(alignment: .leading, spacing: spacing) {
                            // layoutSection describes a render group defined in the render config
                            if fields.count == 0 && edges.count == 0 {
                                // TODO: error when group doesnt exist?

                                renderConfig.render(
                                    item: self.item,
                                    group: groupKey,
                                    arguments: self._args(groupKey: groupKey,
                                                          name: groupKey,
                                                          item: self.item)
                                )
                            }
                            // Render boths fields and edges in the same section
                            else {
                                // Render the fields
                                if fields.count > 0 {
                                    ForEach(fields, id: \.self) { field in
                                        renderConfig.render(
                                            item: self.item,
                                            group: groupKey,
                                            arguments: self._args(groupKey: groupKey,
                                                                  name: field,
                                                                  item: self.item)
                                        )
                                    }
                                }
                                // Render the edges
                                if edges.count > 0 {
                                    ForEach<[Edge], Edge, AnyView>(edges,
                                                                   id: \.self) { edge in
                                        let targetItem = edge.target()
                                        return renderConfig.render(
                                            item: targetItem,
                                            group: groupKey,
                                            arguments: self._args(groupKey: groupKey,
                                                                  value: targetItem,
                                                                  item: targetItem,
                                                                  edge: edge)
                                        )
                                    }
                                }
                            }
                        }
                        .padding(EdgeInsets(top: padding[0],
                                            leading: padding[3],
                                            bottom: padding[2],
                                            trailing: padding[1]))
                    }

                    else if fields.count == 0 && edges.count == 0 {
                        // Error
                    }

                    // Default renderers
                    else {
                        // Render groups with the default render row
                        if fields.count > 0 {
                            ForEach(fields, id: \.self) { field in
                                DefaultGeneralEditorRow(
                                    context: self._context,
                                    item: self.item,
                                    prop: field,
                                    readOnly: readOnly,
                                    isLast: fields.last == field,
                                    renderConfig: renderConfig,
                                    arguments: self._args(name: field,
                                                          value: self.item.get(field),
                                                          item: self.item)
                                )
                            }
                        }
                        // Render lists with their default renderer
                        if edges.count > 0 {
                            ScrollView {
                                VStack(alignment: .leading, spacing: spacing) {
                                    ForEach<[Edge], Edge, ItemCell>(edges, id: \.self) { edge in
                                        let targetItem = edge.target()
                                        return ItemCell(
                                            item: targetItem,
                                            rendererNames: ["generalEditor"],
                                            arguments: self._args(groupKey: groupKey,
                                                                  name: edge.type ?? "",
                                                                  value: targetItem,
                                                                  item: self.item,
                                                                  edge: edge)
                                        )
                                    }
                                }
                                .padding(EdgeInsets(top: padding[0],
                                                    leading: padding[3],
                                                    bottom: padding[2],
                                                    trailing: padding[1]))
                            }
                            .frame(maxHeight: 1000)
                            .fixedSize(horizontal: false, vertical: true)
                        }
                    }

                    if dividers { Divider() }
                }
            }
        )
    }

    func getProperties(_ item: ItemRecord, _ used: [String]) -> [String] {
        item.objectSchema.properties.filter {
            !used.contains($0.name)
                && !$0.isArray
        }.map(\.name)
    }

    func _args(
        groupKey: String = "",
        name: String = "",
        value _: Any? = nil,
        item: ItemRecord?,
        edge: Edge? = nil
    ) -> CVUViewArguments {
        
        CVUViewArguments(args: <#T##[String : CVUValue]#>, argumentItem: <#T##ItemRecord?#>, parentArguments: <#T##CVUViewArguments?#>)
        ViewArguments(
            [
                "subject": CVUValue.constant(CVUValue.Constant.item),
                "readOnly": CVUValue.constant(!context.editMode),
                "title": CVUValue.constant(groupKey.camelCaseToWords().uppercased()),
                "displayName": name.camelCaseToWords().capitalizingFirst(),
                "name": name,
                "edge": edge,
                ".": item,
            ],
            renderConfig.viewArguments?.cascadeStack
        )
    }

    func getAction(edgeType: String, itemType: String) -> Action {
        ActionOpenViewByName(
            context,
            values: [
                "viewName": "choose-item-by-query",
                "viewArguments": ViewArguments([
                    "query": itemType,
                    "type": edgeType,
                    "subject": item,
                    "renderer": "list",
                    "edgeType": edgeType,
                    "title": "Choose a \(itemType)",
                    "item": item,
                ]),
                "icon": "plus",
                "renderAs": RenderType.popup,
            ]
        )
    }

    struct SectionStyle {
        let title: String?
        let dividers: Bool?
        let showTitle: Bool?
        let action: Action?
        let spacing: CGFloat?
        let padding: [CGFloat]
    }

    func sectionStyle(_ groupKey: String) -> SectionStyle {
        let s = renderConfig.getGroupOptions(groupKey)
        let allPadding = getValue(groupKey, s["padding"] as Any?, CGFloat.self) ?? 0

        return SectionStyle(
            title: getValue(groupKey, s["title"] as Any?, String.self)?.uppercased(),
            dividers: getValue(groupKey, s["dividers"] as Any?, Bool.self),
            showTitle: getValue(groupKey, s["showTitle"] as Any?, Bool.self),
            action: getValue(groupKey, s["action"] as Any?, Action.self),
            spacing: getValue(groupKey, s["spacing"] as Any?, CGFloat.self),
            padding: getValue(groupKey, s["padding"] as Any?, [CGFloat].self)
                ?? [allPadding, allPadding, allPadding, allPadding]
        )
    }

    func getValue<T>(_ groupKey: String, _ value: Any?, _: T.Type = T.self) -> T? {
        if value == nil { return nil }

        if let expr = value as? Expression {
            let args = _args(groupKey: groupKey, name: groupKey, item: item)
            do {
                return try expr.execForReturnType(T.self, args: args)
            }
            catch {
                debugHistory.error("\(error)")
                return nil
            }
        }

        return value as? T
    }
}


struct DefaultGeneralEditorRow: View {
    @EnvironmentObject var context: MemriContext

    var item: ItemRecord
    var prop: String
    var readOnly: Bool
    var isLast: Bool
    var renderConfig: CascadingRendererConfig
    var arguments: ViewArguments

    var body: some View {
        // Get the type from the schema, because when the value is nil the type cannot be determined
        let propType = item.objectSchema[prop]?.type
        let propValue: Any? = self.item.get(self.prop)

        return VStack(spacing: 0) {
            if propValue == nil && readOnly {
                EmptyView()
            }
            else {
                VStack(alignment: .leading, spacing: 4) {
                    Text(prop
                        .camelCaseToWords()
                        .lowercased()
                        .capitalizingFirst())
                        .generalEditorLabel()

                    if renderConfig.hasGroup(prop) {
                        renderConfig.render(item: item, group: prop, arguments: arguments)
                    }
                    else if readOnly {
                        if [.string, .bool, .date, .int, .double].contains(propType) {
                            defaultRow(ExprInterpreter.evaluateString(propValue, defaultValue: ""))
                        }
                        else if propType == .object {
                            if propValue is ItemRecord {
                                MemriButton(item: propValue as? ItemRecord)
                                    .environmentObject(self.context)
                            }
                            else {
                                defaultRow()
                            }
                        }
                        else { defaultRow() }
                    }
                    else {
                        switch propType {
                        case .string:
                            stringRow()
                        case .bool:
                            boolRow()
                        case .date:
                            dateRow()
                        case .int:
                            intRow()
                        case .double:
                            doubleRow()
                        default:
                            defaultRow()
                        }
                    }
                }
                .fullWidth()
                .padding(.bottom, 10)
                .padding(.horizontal)
                .background(Color(.systemBackground))

                if !isLast {
                    Divider().padding(.leading, 35)
                }
            }
        }
    }

    func stringRow() -> some View {
        let binding = Binding<String>(
            get: { self.item.getString(self.prop) ?? "" },
            set: {
                self.item.set(self.prop, $0)
            }
        )

        return MemriTextField(value: binding,
                              clearButtonMode: .whileEditing,
                              isEditing: $context.editMode,
                              isSharedEditingBinding: true)
            .generalEditorCaption()
    }

    func boolRow() -> some View {
        let binding = Binding<Bool>(
            get: { self.item[self.prop] as? Bool ?? false },
            set: { _ in
                do {
                    try self.item.toggle(self.prop)
                    self.context.objectWillChange.send()
                }
                catch {}
            }
        )

        return Toggle(isOn: binding) {
            Text(prop
                .camelCaseToWords()
                .lowercased()
                .capitalizingFirst())
        }
        .generalEditorCaption()
    }

    func intRow() -> some View {
        let binding = Binding<Int>(
            get: { self.item[self.prop] as? Int ?? 0 },
            set: {
                self.item.set(self.prop, $0)
                self.context.objectWillChange.send()
            }
        )

        return MemriTextField(value: binding,
                              clearButtonMode: .whileEditing,
                              isEditing: $context.editMode,
                              isSharedEditingBinding: true)
            .generalEditorCaption()
    }

    func doubleRow() -> some View {
        let binding = Binding<Double>(
            get: { self.item[self.prop] as? Double ?? 0 },
            set: {
                self.item.set(self.prop, $0)
                self.context.objectWillChange.send()
            }
        )

        return MemriTextField(value: binding,
                              clearButtonMode: .whileEditing,
                              isEditing: $context.editMode,
                              isSharedEditingBinding: true)
            .generalEditorCaption()
    }

    @ViewBuilder
    func dateRow() -> some View {
        let binding = Binding<Date>(
            get: { self.item[self.prop] as? Date ?? Date() },
            set: {
                self.item.set(self.prop, $0)
                self.context.objectWillChange.send()
            }
        )

        DatePicker("", selection: binding, displayedComponents: .date)
            .labelsHidden()
    }

    func defaultRow(_ caption: String? = nil) -> some View {
        Text(caption ?? prop.camelCaseToWords().lowercased().capitalizingFirst())
            .generalEditorCaption()
    }
}

public extension View {
    func generalEditorLabel() -> some View { modifier(GeneralEditorLabel()) }
    func generalEditorCaption() -> some View { modifier(GeneralEditorCaption()) }
    func generalEditorHeader() -> some View { modifier(GeneralEditorHeader()) }
    func generalEditorInput() -> some View { modifier(GeneralEditorInput()) }
}

private struct GeneralEditorInput: ViewModifier {
    func body(content: Content) -> some View {
        content
            .fullHeight()
            .font(.system(size: 16, weight: .regular))
            .padding(10)
//            .border(width: [0, 0, 1, 1], color: Color(hex: "#eee"))
            .generalEditorCaption()
    }
}

private struct GeneralEditorLabel: ViewModifier {
    func body(content: Content) -> some View {
        content
            .foregroundColor(Color(hex: "#38761d"))
            .font(.system(size: 14, weight: .regular))
            .padding(.top, 10)
    }
}

private struct GeneralEditorCaption: ViewModifier {
    func body(content: Content) -> some View {
        content
            .font(.system(size: 18, weight: .regular))
            .foregroundColor(Color(hex: "#223322"))
    }
}

private struct GeneralEditorHeader: ViewModifier {
    func body(content: Content) -> some View {
        content
            .font(Font.system(size: 15, weight: .regular))
            .foregroundColor(Color(hex: "#434343"))
            .padding(.bottom, 5)
            .padding(.top, 24)
            .padding(.horizontal)
            .foregroundColor(Color(hex: "#333"))
    }
}
