import SwiftUI
import ASCollectionView


struct GridRendererLayout_PhotoGrid: GridRendererLayout {
    init(renderConfigResolver: CVUPropertyResolver) {
        self.resolver = renderConfigResolver
    }
    
    var resolver: CVUPropertyResolver
    
    let layoutName = "photoGrid"
    
    /// A value that changes when the layout needs to be reloaded
    var layoutStateID: String { layoutName }
    
    var layout: ASCollectionLayout<Int> {
        ASCollectionLayout(scrollDirection: .vertical, interSectionSpacing: 0) {
            ASCollectionLayoutSection { environment in
                let contentInset = resolver.nsEdgeInset ?? .init(top: 10, leading: 10, bottom: 10, trailing: 10)
                let columns = 3
                let spacing = resolver.spacing ?? .zero
                
                let singleBlockSize = (environment.container.effectiveContentSize.width
                                        - contentInset.leading - contentInset.trailing
                                        - spacing.x * CGFloat(columns - 1)) / CGFloat(columns)
                func gridBlockSize(forSize size: Int, sizeY: Int? = nil) -> NSCollectionLayoutSize {
                    let x = CGFloat(size) * singleBlockSize + spacing.x * CGFloat(size - 1)
                    let y = CGFloat(sizeY ?? size) * singleBlockSize
                        + spacing.y * CGFloat((sizeY ?? size) - 1)
                    return NSCollectionLayoutSize(
                        widthDimension: .absolute(x),
                        heightDimension: .absolute(y)
                    )
                }
                let itemSize = gridBlockSize(forSize: 1)
                
                let item = NSCollectionLayoutItem(layoutSize: itemSize)
                
                let verticalGroupSize = gridBlockSize(forSize: 1, sizeY: 2)
                let verticalGroup = NSCollectionLayoutGroup.vertical(
                    layoutSize: verticalGroupSize,
                    subitem: item,
                    count: 2
                )
                verticalGroup.interItemSpacing = .fixed(spacing.y)
                
                let featureItemSize = gridBlockSize(forSize: 2)
                let featureItem = NSCollectionLayoutItem(layoutSize: featureItemSize)
                
                let fullWidthItemSize = gridBlockSize(forSize: 3, sizeY: 1)
                let fullWidthItem = NSCollectionLayoutItem(layoutSize: fullWidthItemSize)
                
                let verticalAndFeatureGroupSize = gridBlockSize(forSize: 3, sizeY: 2)
                let verticalAndFeatureGroupA = NSCollectionLayoutGroup.horizontal(
                    layoutSize: verticalAndFeatureGroupSize,
                    subitems: [verticalGroup, featureItem]
                )
                verticalAndFeatureGroupA.interItemSpacing = .fixed(spacing.x)
                let verticalAndFeatureGroupB = NSCollectionLayoutGroup.horizontal(
                    layoutSize: verticalAndFeatureGroupSize,
                    subitems: [featureItem, verticalGroup]
                )
                verticalAndFeatureGroupB.interItemSpacing = .fixed(spacing.x)
                
                let rowGroupSize = gridBlockSize(forSize: 3, sizeY: 1)
                let rowGroup = NSCollectionLayoutGroup.horizontal(
                    layoutSize: rowGroupSize,
                    subitem: item,
                    count: Int(columns)
                )
                rowGroup.interItemSpacing = .fixed(spacing.x)
                
                let outerGroupSize = gridBlockSize(forSize: 3, sizeY: 7)
                let outerGroup = NSCollectionLayoutGroup.vertical(
                    layoutSize: outerGroupSize,
                    subitems: [
                        verticalAndFeatureGroupA,
                        rowGroup,
                        fullWidthItem,
                        verticalAndFeatureGroupB,
                        rowGroup,
                    ]
                )
                outerGroup.interItemSpacing = .fixed(spacing.y)
                
                let section = NSCollectionLayoutSection(group: outerGroup)
                section.contentInsets = contentInset
                section.interGroupSpacing = 1
                return section
            }
        }
    }
}
