//
// MapRenderer.swift
// Copyright © 2020 memri. All rights reserved.

import Combine
import Foundation
import SwiftUI

/// The map renderer
/// This presents the data items on a map
/// - for an item to be shown the CVU for ItemType > map {...} must define an expression resolving to either a `Location` or `Address` item
/// - you can provide a `label` property in the CVU for ItemType > map {...}
struct MapRendererView: View {
    @ObservedObject var viewContext: ViewContextController
    @EnvironmentObject var sceneController: SceneController

    var mapConfig: MapViewConfig {
        let resolver = viewContext.rendererDefinitionPropertyResolver
        let moveable = resolver.bool("moveable", defaultValue: true)
        
        return MapViewConfig(
            dataItems: viewContext.items,
            locationResolver: {
                viewContext.nodePropertyResolver(for: $0)?.items("location") ?? []
            },
            addressResolver: {
                viewContext.nodePropertyResolver(for: $0)?.items("address") ?? []
            },
            labelResolver: {
                viewContext.nodePropertyResolver(for: $0)?.string("label")
            },
            moveable: moveable,
            onPress: { item in
                if let press = viewContext.nodePropertyResolver(for: item)?.action(key: "onPress") {
                    press.execute(sceneController: sceneController, context: viewContext.getCVUContext(forItem: item))
                }
            }
        )
    }
    
    var body: some View {
        MapView(config: mapConfig)
            .background(Color(.secondarySystemBackground))
    }
}

//struct MapRendererView_Previews: PreviewProvider {
//    static var previews: some View {
//        MapRendererView()
//    }
//}
