//
//  SceneContentView.swift
//  Memri
//
//  Created by T Brennan on 23/1/21.
//

import SwiftUI

/// This view is used to display the contents of a ViewContextController (ie a single screen in the navigation stack)
struct SceneContentView: View {
    @EnvironmentObject var sceneController: SceneController
    @ObservedObject var context: ViewContextController
    
    /// Keep track of whether the search bar is currently open (keyboard shown)
    @State var searchBarOpen: Bool = false
    
    /// Translates the rendererName to the correct Renderer view
    @ViewBuilder
    var renderer: some View {
        switch context.config.rendererName.lowercased() {
        case "list":
            ListRendererView(viewContext: context)
        case "grid":
            GridRendererView(viewContext: context)
        case "map":
            MapRendererView(viewContext: context)
        case "timeline":
            TimelineRendererView(viewContext: context)
        case "calendar":
            CalendarRendererView(viewContext: context)
        case "photoviewer":
            PhotoViewerRendererView(viewContext: context)
        case "chart":
            ChartRendererView(viewContext: context)
        case "singleitem":
            SingleItemRendererView(viewContext: context)
        case "noteeditor":
            NoteEditorRendererView(viewContext: context)
        case "labelannotation":
            LabelAnnotationRendererView(viewContext: context)
        case "fileviewer":
            FileRendererView(viewContext: context)
        default:
            Text("No renderer selected").bold().frame(maxWidth: .infinity, maxHeight: .infinity)
        }
    }
    
    var body: some View {
        VStack(spacing: 0) {
            renderer
            if searchBarOpen {
                SearchView(context: context, isActive: $searchBarOpen)
            } else {
                BottomBarView(context: context,
                              onSearchPressed: { searchBarOpen = true },
                              onFilterButtonPressed: {
                                withAnimation {
                                    sceneController.filterPanelIsVisible = true
                                }
                              })
            }
        }
        .onAppear {
            context.onAppear()
        }
        .onDisappear {
            context.onDisappear()
        }
    }
}
