//
//  ViewContext.swift
//  memri
//
//  Created by T Brennan on 7/12/20.
//  Copyright © 2020 memri. All rights reserved.
//

import Foundation
import Combine

/// This type is used to hold all the configuration necessary to display a screen.
/// It is stored as part of the navigation stack, and the whole stack is persisted to the database
struct ViewContext: Equatable, Codable {
    var viewName: String?

    var rendererName: String

    var viewDefinition: CVUDefinitionContent = CVUDefinitionContent()

    var query: DatabaseQueryConfig

    var viewArguments: CVUViewArguments?

    var focusIndex: Int = 0
}

/// A class type that holds a ViewContext struct. This allows the struct to be shared between objects while maintaining value semantics.
class ViewContextHolder: Codable, Equatable {
    var config: ViewContext {
        didSet {
            configPublisher.send()
        }
    }
    
    init(_ config: ViewContext) {
        self.config = config
    }
    
    required init(from decoder: Decoder) throws {
        let container = try decoder.singleValueContainer()
        config = try container.decode(ViewContext.self)
    }
    func encode(to encoder: Encoder) throws {
        var container = encoder.singleValueContainer()
        try container.encode(config)
    }
    
    static func == (lhs: ViewContextHolder, rhs: ViewContextHolder) -> Bool {
        lhs.config == rhs.config
    }
    
    let configPublisher = PassthroughSubject<Void, Never>()
}
