//
//  DatabaseQueryTests.swift
//  MemriDatabaseTests
//
//  Created by T Brennan on 16/12/20.
//

import XCTest
@testable import Memri

class DatabaseQueryTests: XCTestCase {
    let databaseController = DatabaseController()
    let schema = try! Schema.loadFromFile()
    
    override func setUpWithError() throws {
        // Put setup code here. This method is called before the invocation of each test method in the class.
        try databaseController.databasePool.erase()
        try databaseController.setupDatabase()
        try DemoData.importDemoData(databaseController: databaseController)
    }
    
    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }

    func testQuery() throws {
        let queryDef = DatabaseQueryConfig(itemTypes: ["Note", "Photo"],
                                           itemUIDs: [StringUUID(uid: "testUID1"), StringUUID(uid: "testUID2")],
                                           dateModifiedRange: nil,
                                           dateCreatedRange: nil,
                                           pageSize: 1000,
                                           currentPage: 0)
        try databaseController.read { db in
            let result = try queryDef.executeRequest(db: db)
            print(result.count)
        }
    }
    
    func testQueryPerformance() throws {
        self.measure {
            try! testQuery()
        }
    }
    
    
    func testQueryWithCondition() throws {
        let queryDef = DatabaseQueryConfig(itemTypes: [],
                                           itemUIDs: [],
                                           dateModifiedRange: nil,
                                           dateCreatedRange: nil,
                                           pageSize: 1000,
                                           currentPage: 0,
                                           conditions: [.propertyEquals(.init(name: "title", value: "A demo note"))])
        try databaseController.read { db in
            let result = try queryDef.executeRequest(db: db)
            XCTAssertEqual(result.count, 1)
        }
    }
    
    func testQueryWithConditionPerformance() throws {
        self.measure {
            try! testQueryWithCondition()
        }
    }
    
    func testWriteInRead() throws {
        try databaseController.read { db in
            guard let result = try databaseController.search(string: "trailhead first").first else { return }
            try result.setPropertyValue(name: "content", value: .string("TESTING 123"), db: databaseController)
        }
    }
}
